''
'' chained 64 bit timer/counter
''
''        Author: Marko Lukat
'' Last modified: 2011/02/07
''       Version: 0.6
''
CON
  _clkmode = client#_clkmode
  _xinfreq = client#_xinfreq

OBJ
  client: "core.con.client.demoboard"
  thread: "core.aux.thread"

  serial: "FullDuplexSerial"
  
PUB init | time[2]

  serial.start(31, 30, %0000, 115200)                   ' debug output

  ctrx_init_pasm                                        ' setup counter(s)
  
  repeat
    serial.rxflush                                      ' purge input buffer
    serial.rx                                           ' wait for key
    serial.tx(0)                                        ' clear screen

    ctrx_read_pasm(@time{0}, constant(heap +1))         ' read count
    
    serial.hex(time[1], 8)                              '
    serial.hex(time{0}, 8)                              ' display it

CON
  ctrx_pin = 16                                         ' link pin number
  
VAR
  long  command                                         ' overlay command listener
  
PRI ctrx_init_pasm

  if result := thread.create(-1, @command, TRUE, TRUE)  ' launch overlay handler
    command := thread.encodeLoadSE(@timer, @mask)       ' load and setup overlay
    repeat while command                                ' wait for result

PRI ctrx_read_pasm(parameter, cmd)

  cmd.word[1] := parameter                              ' merge
  command := cmd                                        ' run overlay
  repeat while command                                  ' wait for result

CON
  addr = thread#regADDR                                 ' overlay command
  done = thread#cmdDONE                                 ' return target
  heap = thread#cmdHEAP                                 ' user area

DAT             org     heap                            ' 64 bit counter overlay

timer           jmpret  $, #:setup                      ' once

:read           mov     cntH, phsa                      ' adjustment is 4-1-1
                mov     cntL, phsb                      ' 4: instruction delay
'               cmp     cntL, #6 wc                     ' 1: A1
'       if_c    add     cntH, frqa                      ' 1: enable->increment
'
' In the ideal case cntL >= 6 means cntH caught the NEGEDGE and has been
' incremented. As this is the real world we allow for 6 extra cycles and
' re-read both counters if cntL is smaller, i.e. 0..11.
'
                cmp     cntL, #12 wc                    ' check for DMZ
        if_c    jmp     #:read                          ' read again

                shr     addr, #16                       ' @result{0} (addr:cmd = 16:16)
                wrlong  cntL, addr                      ' lower 32 bit
                add     addr, #4                        ' @result[1]
                wrlong  cntH, addr                      ' upper 32 bit

                jmp     $000                            ' ret (jmp #done)

' Single pulse coupling is not safe, edge is better but this may still
' introduce an N cycle error (depending on when the edge is detected).

' Best practice would probably be edge and a DMZ around the switch point
' for example +/- 10 cycles (we can always blame hub window sync).

:setup          andn    outa, mask                      ' make space
                or      dira, mask                      ' drive link pin

                movs    ctra, #ctrx_pin                 ' link pin
                movi    ctra, #%0_01110_000             ' NEGEDGE
                movs    ctrb, ctra                      ' link pin
                movi    ctrb, #%0_00100_000             ' NCO single-ended

                mov     frqb, #1                        ' |
                mov     frqa, #1                        ' | enable

                mov     phsb, #0                        ' |
                mov     phsa, #0                        ' | preset
                
                jmp     $000                            ' ret (jmp #done)

' initialised data and/or presets
                
mask            long    |< ctrx_pin                     ' pin mask

' uninitialised data and/or temporaries

cntH            res     1                               ' upper 32 bit
cntL            res     1                               ' lower 32 bit

                fit
                
DAT